library(tidyverse) # dplyr, tidyr, ggplot2 and friends

########################################################
# Data wrangling
########################################################

# Resources:
# * <https://dplyr.tidyverse.org/>
# * <http://r4ds.had.co.nz/transform.html>

# Focuses on when data is given. Can be entered manually:
d1 <- tibble(w = mtcars$wt[1:2], h = mtcars$hp[1:2])
d1

########################################################
# tibbles
########################################################
# Modern data frames that work well with `tidyverse`.
# <https://cran.r-project.org/web/packages/tibble/vignettes/tibble.html>

# Versus "vanilla"/base R:
data.frame(w = mtcars$wt, h = mtcars$hp) # base R
tibble(w = mtcars$wt, h = mtcars$hp) # tidyverse

d2 <- tribble(
  ~x, ~y,
   1,  2,
   3,  4
)
d2

# Glue:
bind_cols(d1, d2)
bind_rows(d1, d1)

# Dataframes vs tibbles
mtcars
as_tibble(mtcars)


########################################################
# Data types
########################################################
# * `int` stands for integers.
# * `dbl` stands for doubles, or real numbers.
# * `chr` stands for character vectors, or strings.
# * `fctr` stands for factors (categorical variables).
# * `dttm` stands for date-times (a date + a time).
# * Others: `lgl`, `date`

########################################################
# The pipe operator, `%>%`
########################################################
filter(mpg, hwy >= 20, hwy <= 26)
select(filter(mpg, hwy >= 20, hwy <= 26), manufacturer, hwy)

# * Each data handling operation performed results in one more level of nested calls. 
# * The code will quickly become difficult to read
#    * Read it from the inner most call and out 
#    * Operation description (function name) separated from arguments

#          .-'-.
#        /`     |__
#      /`  _.--`-,-`
#      '-|`   a '<-.   []
#        \     _\__) \=`
#         C_  `    ,_/
#           | ;----'
#      _.---| |--._
#    .'  _./' '\._ '.
#   /--'`  `-.-`  `'-\
#  |         o        \
#  |__ .             / )
# (___)|     o     \/ /
#  | | |           |-'\     .--.
# /  |_.-"""-.-'=_ |   '--./ = .`
#<   `  =-->    _= /        __/
# '._     _.-"-.= /`"-...-"` 
#    `;"""`     __/
#      /```````` |
#     |     ,     \
#     \      \     \
#      \      \     \
#jgs   |       \     \
# (Art by Joan Stark)

# %>% : The pipe operator
# RStudio: CTRL + SHIFT + M

filter(mpg, hwy >= 20, hwy <= 26)
mpg %>% filter(hwy >= 20, hwy <= 26)

# Known many years in functional programming, unix terminals etc.

# A crucial design decision of the `tidyverse`:
# * First argument is a `tibble` (or `data.frame`)
# * Subsequent arguments say what to do with the `tibble` (or `data.frame`)
# * Always return a `tibble` (or `data.frame`)

# This enables the so-called piping operator, `%>%`. Example:

# f(x, ...) === x %>% f(...)
sum(sqrt(1:10))
1:10 %>% sqrt() %>% sum()

filter(mpg, hwy >= 20, hwy <= 26) %>% 
  select(manufacturer, hwy)

mpg %>% 
  filter(hwy >= 20, hwy <= 26) %>% 
  select(manufacturer, hwy)

# %>% : The pipe operator
# RStudio: CTRL + SHIFT + M
# Read as: "and then"

########################################################
# Operations
########################################################

# * `filter()`: keep rows matching criteria
# * `select()`: pick/reorder columns by name
# * `rename()`: rename columns by name
# * `arrange()`: reorder rows
# * `mutate()`: add new or modify existing variables
# * `summarise()`: perform aggregation operations
# * `group_by()`: introduce aggregation groupings
# * `count()`: a short-hand for `group_by()` and `summarise()`

# ... and a ton of helper functions.

# Examples:

mpg %>% select(manufacturer, hwy)

mpg %>% select(class, everything())

mpg %>% select(-manufacturer, -hwy)

count(mpg, class)

mpg %>% count(class)
mpg %>% 
  group_by(class) %>% 
  summarise(n())
mpg %>% 
  group_by(class) %>% 
  summarise(n = n())

mpg %>% 
  group_by(class) %>% 
  summarise(
    n = n(),
    displ = mean(displ)) %>% 
  arrange(desc(n))

mpg %>% 
  group_by(manufacturer) %>% 
  summarise(mean_hwy = mean(hwy)) %>% 
  arrange(desc(mean_hwy))

mpg %>% 
  group_by(cyl, trans) %>% 
  summarise(mean_hwy = mean(hwy)) %>% 
  arrange(cyl, desc(mean_hwy))

# Comment out lines, tip: add identity()/I():
mpg %>% 
  group_by(cyl, trans) %>% 
  summarise(mean_hwy = mean(hwy)) %>% 
  arrange(cyl, desc(mean_hwy)) %>% 
  identity()

mpg %>% 
  group_by(cyl, trans) %>% 
  summarise(mean_hwy = mean(hwy)) %>% 
  arrange(cyl, desc(mean_hwy)) %>% 
  I()


mpg %>% 
  group_by(class, displ) %>% 
  summarise(mean_hwy = mean(hwy))

ggplot(mpg, aes(displ, color = class)) + 
  geom_density()

d_tmp <- mpg %>% 
  group_by(class) %>% 
  summarise(displ = mean(displ))
d_tmp

ggplot(mpg, aes(displ, color = class)) + 
  geom_density() +
  geom_vline(data = d_tmp, aes(xintercept = displ, color = class))

ggplot(mpg, aes(displ, color = class)) + 
  geom_density() +
  geom_vline(data = d_tmp, aes(xintercept = displ, color = class)) +
  facet_wrap(~ class)


########################################################
# Long/wide
########################################################

# Wide to long
mpg

mpg %>% 
  select(class, hwy, cty)

mpg %>% 
  select(class, hwy, cty) %>% 
  gather(type, value, -class) # gather(type, value, hwy, cty)

# ... %>% gather(new_col_name, new_col_value, (columns_of_same_type_to_be_converted))

mpg %>% 
  select(class, hwy, cty) %>% 
  gather(type, value, -class) %>% 
  ggplot(aes(type, value, color = class)) + 
    geom_boxplot()

d_tmp <- mpg %>% 
  select(class, hwy, cty) %>% 
  gather(type, value, -class)
d_tmp
ggplot(d_tmp, aes(type, value, color = class)) + 
  geom_boxplot()

# Long to wide

mpg

mpg %>% 
  mutate(drv = paste0("drv_", drv)) %>% 
  select(class, drv, hwy)

d_tmp <- mpg %>% 
  mutate(drv = paste0("drv_", drv)) %>% 
  select(class, drv, hwy) %>% 
  group_by(drv, class) %>% 
  summarise(hwy = mean(hwy))
d_tmp

d_tmp %>% 
  spread(drv, hwy)

d_tmp %>% 
  spread(drv, hwy, fill = 0)

# ... %>% spread(col_with_new_col_names, 
#                col_with_new_col_values)

# Exercises
# http://r4ds.had.co.nz/transform.html:
# R4DS (online): 

# Run
library(nycflights13)
library(tidyverse)

# Read Section 5.1
# 5.2.4 Exercises 1-3
# 5.3.1 Exercises
